
/* Control Board Test Commands */

#define ID_REQ		0x01			/* request test version id */
#define RAM_TEST	0x02			/* ram test */
#define EEROM_TEST	0x03			/* eerom test */
#define SWITCH_TEST	0x04			/* switch test */
#define	MARK_TEST1	0x05			/* 1st synthesizer mark test */
#define MARK_TEST2	0x06			/* 2nd synthesizer mark test */
#define SPACE_TEST1	0x07			/* 1st synthesizer space test */
#define	SPACE_TEST2	0x08			/* 2nd synthesizer space test */
#define MCLK_TEST1	0x09			/* 1st synthesizer mark clock test */
#define MCLK_TEST2	0x0a			/* 2nd synthesizer mark clock test */
#define SCLK_TEST1	0x0b			/* 1st synthesizer space clock test */
#define SCLK_TEST2	0x0c			/* 2nd synthesizer space clock test */
#define HSCLK_TEST1	0x0d			/* 1st synthesizer high speed clock test */
#define HSCLK_TEST2	0x0e			/* 2nd synthesizer high speed clock test */
#define	SANITY_TEST	0x0f			/* sanity timer test */
#define CONTROL_TEST	0x10			/* control signal test */
#define STAT_TEST	0x11			/* status register test */
#define NMI_TEST	0x12			/* nmi timer test */

#define	CHECK_SUM	0x1f			/* request kchecksum value comand */

/* Front Panel Test Commands */

#define	LED_TEST	0x20			/* led test */
#define SEGMENT_TEST	0x21			/* seven segment display test */
#define KEYPAD_TEST	0x22			/* keypad test */
#define BAR_TEST	0x23			/* bar graph test */
#define ALLON_TEST	0x24			/* all segments on test */

/* Modem Board Test Commands */


#define STOP_TEST	0xff			/* end current test command */


/* test menu screen locations */

#define HEADER_L	19
#define HEADER_T	5			/* test version number screen location */
#define VERSION_L	34			/* test version number display location */

#define	MENU1_TOP	10			/* top of main options menu */
#define	MENU1_LEFT	19			/* left side of main options menu */

#define	CMENU_TOP	8			/* top of control board tests */
#define CMENU_LEFT	19			/* left side of control board tests */

#define	MMENU_TOP	8			/* top of modem board tests */
#define MMENU_LEFT	19			/* left side of modem board tests */

#define	FPMENU_TOP	10			/* top of front panel tests */
#define FPMENU_LEFT	19			/* left side of front panel tests */

#define DMENU_TOP	10			/* top of debug menu options */
#define DMENU_LEFT	19			/* left side of debug menu options */

/* Test Window character definitions */

#define DULEFT_CHAR	0xd5			/* double line upper left corner */
#define DHORZ_CHAR	0xcd			/* double line horizontal character */
#define DURIGHT_CHAR	0xb8			/* double line upper right corner */
#define VERT_CHAR	0xb3			/* single line verticle character */
#define DLLEFT_CHAR	0xd4			/* double line lower left corner */
#define DLRIGHT_CHAR	0xbe			/* double line lower right corner */


/* Test option window size definitions */

#define RAM_L		19
#define RAM_R		47
#define RAM_T		11			/* ram test option window */
#define RAM_B		17

#define EROM_L		19
#define EROM_R		50
#define	EROM_T		12			/* eerom test option window */
#define EROM_B		18

#define SWITCH_L	19
#define SWITCH_R	60
#define SWITCH_T	13			/* switch test option window */
#define SWITCH_B	20

#define SYN_L		19
#define SYN_R		52
#define SYN_T		14			/* synthesizer test option window */
#define SYN_B		20

#define SANITY_L	19
#define SANITY_R	47
#define SANITY_T	15			/* sanity timer test option window */
#define SANITY_B	21

#define CONTROL_L	19
#define CONTROL_R	56
#define CONTROL_T	16			/* control signal test option window */
#define CONTROL_B	20

#define STATUS_L	19
#define STATUS_R	65
#define STATUS_T	17			/* status register test option window */
#define STATUS_B	24

#define NMI_L		19
#define NMI_R		56
#define NMI_T		18			/* nmi timer test option window */
#define NMI_B           24

#define LED_L		19
#define LED_R		48
#define LED_T		13			/* led test option window */
#define LED_B		17

#define SEG_L		19
#define SEG_R		53
#define SEG_T		14			/* seven segment test option window */
#define SEG_B		18

#define KEYPAD_L	19
#define KEYPAD_R	47
#define KEYPAD_T	15			/* keypad test option window */
#define KEYPAD_B	21

#define	BAR_L		19
#define BAR_R		48
#define BAR_T		16			/* bar graph display test window */
#define BAR_B		20

#define ALL_L		19
#define ALL_R		51
#define ALL_T		17			/* all segments on test window */
#define ALL_B		21

#define MB_L		19
#define MB_R		67
#define	MB_T		14			/* modem board test window */
#define MB_B		20

#define IDX		60			/* test step id number x position */
#define IDY		15			/* test step id number y position */


/* com port parameters */

#define BAUD_9600  	12
#define DBITS_8    	0x03
#define SBITS_2    	0x04
#define NO_PARITY  	0x00
#define DATA_READY 	0x01			/* com1 data ready */
#define	TX_READY	0x20			/* com1 transmit ready */
#define COM1       	0x3f8
#define IEN        	0x01
#define LCR        	0x03
#define MCR        	0x04
#define LSR        	0x05

#define CR		0x0d                    /* carriage return character */
#define	SP		0x20			/* space bar keycode */
#define BS		0x08			/* back space keycode */
#define ESC		0x1b			/* escape key keycode */

/* command strings */

extern	char	TC1[46];				/* test command 1 */
extern	char	TC2[22];				/* test command 2 */

extern	char	T0[4];					/* t0 command bit mode external */
extern	char	T5[4];					/* t5 command bit mode internal 0dBm */
extern	char	T6[4];					/* t6 command bit mode internal -20dBm */
extern	char	T7[4];					/* t7 command bit mode internal -45dBm */

extern	char	C01[5];					/* c01 command channel 1 select */
extern	char	C02[5];					/* c02 command channel 2 select */

extern	char	P1[4];					/*12 P1 command copy chaneel 1 settings to channel 2 */

extern	char	U1[4];					/* u1 command mute on */

extern 	char	A0[4];					/* a0 command amh off */

extern	char	W0[4];					/* w0 command synchronous mode */

extern	char	J0[4];					/* j0 fsk command */
extern	char	J1[4];					/* j1 mark only command */
extern  char	J2[4];					/* j2 space only command */

extern	char	D1[4];					/* d1 command div mode */

extern	char	B30[5];
extern	char	B75[5];
extern	char	B86[5];
extern	char	B114[6];
extern	char	B145[6];
extern	char	B170[6];
extern	char	B202[6];
extern	char	B253[6];				/* baud rate commands */
extern	char	B327[6];
extern	char	B337[6];
extern	char 	B443[6];
extern	char 	B540[6];
extern	char	B625[6];
extern	char	B689[6];
extern	char	B760[6];
extern	char	B854[6];
extern	char	B952[6];
extern	char 	B1200[7];

extern	char	S2000[7];
extern	char	M3000[7];

extern	int	result_x;
extern 	int	result_y;						/* test result display pointers */

extern	char	buf[2000];						/* screen restore buffer */

extern	int	cur_test;						/* modem board main test pointer */
extern 	int	sub_test;						/* modem board sub test pointer */
